// DDR2_SDRAMtest_tb.v (DDR2 SDRAMP̃eXgj
`default_nettype none
`timescale 1ps / 1ps

module DDR2_SDRAMtest_tb;

	`include "ddr2_parameters.vh"
	`include "./ddr2_cont_parameters.vh"


	wire [DQ_BITS-1:0] dq;
	wire [DQS_BITS-1:0] dqs;
	reg reset;
	wire [DQS_BITS-1:0] ddr2_dqs_fpga, ddr2_dqs_sdram;
	wire [DQS_BITS-1:0] ddr2_dqs_n_fpga, ddr2_dqs_n_sdram;
	wire [DQ_BITS-1:0] ddr2_dq_fpga, ddr2_dq_sdram;
	reg [DQS_BITS-1:0] ddr2_dqs_fpgan, ddr2_dqs_sdramn;
	reg [DQS_BITS-1:0] ddr2_dqs_n_fpgan, ddr2_dqs_n_sdramn;
	reg [DQ_BITS-1:0] ddr2_dq_fpgan, ddr2_dq_sdramn;
	wire ddr2_clk, ddr2_clkb;
	wire ddr2_cke, ddr2_csb, ddr2_rasb, ddr2_casb, ddr2_web;
	wire [DM_BITS-1:0] ddr2_dm;
	wire [1:0] ddr2_ba;
	wire [ADDR_BITS-1:0] ddr2_address;
	wire reset_b;
	reg enable_o;
	reg clk;
	wire [2:0] cmd;
	reg sdram_clk;
	reg sdram_clkb;
	reg [12:0] sdram_address;
	reg [1:0] sdram_ba;
	reg sdram_cke;
	reg sdram_csb, sdram_rasb, sdram_casb, sdram_web;
	reg [1:0] sdram_dmn;
	wire [1:0] sdram_dm;
	wire sim_clk;
	reg [INTERFACE_DATA_WIDTH-1 : 0] input_data;
	reg [INTERFACE_MASK_WIDTH-1 : 0] input_mask;
	reg read_write;
	wire [INTERFACE_DATA_WIDTH-1 : 0] output_data;
	reg [USER_INPUT_ADDRESS_WIDTH-1 : 0] input_address;
	reg addr_fifo_wren; // AhXpFIFȌ݃Cl[u
	reg wrdata_fifo_wren; // FIFȌ݃Cl[u
	wire addr_fifo_full;
	wire wrdata_fifo_full;
	wire rddata_valid; // ǂݏof[^L
	wire initialize_end;
	
	parameter DELAY_TIME = 3000;
	parameter CLK_PERIOD = 10000;
	
	assign cmd = {ddr2_rasb, ddr2_casb, ddr2_web};
	
	always @(posedge ddr2_clk, posedge reset)
		if (reset)
			enable_o <= 1'b0;
		else
			if (cmd==3'b100)
				enable_o <= 1'b0;
			else if (cmd==3'b101)
				enable_o <= 1'b1;

	always @ *
		if (enable_o == 1'b1)
			ddr2_dqs_fpgan <= #DELAY_TIME ddr2_dqs_sdram;
		else
			ddr2_dqs_fpgan <= #DELAY_TIME {DQS_BITS{1'bz}};
	
	always @ *
		if (enable_o == 1'b1)
			ddr2_dqs_n_fpgan <= #DELAY_TIME ddr2_dqs_n_sdram;
		else
			ddr2_dqs_n_fpgan <= #DELAY_TIME {DQS_BITS{1'bz}};
	
	always @ *
		if (enable_o == 1'b1)
			ddr2_dq_fpgan <= #DELAY_TIME ddr2_dq_sdram;
		else
			ddr2_dq_fpgan <= #DELAY_TIME {DQ_BITS{1'bz}};

	always @ *
		if (enable_o == 1'b0)
			ddr2_dqs_sdramn <= #DELAY_TIME ddr2_dqs_fpga;
		else
			ddr2_dqs_sdramn <= #DELAY_TIME {DQS_BITS{1'bz}};
	
	always @ *
		if (enable_o == 1'b0)
			ddr2_dqs_n_sdramn <= #DELAY_TIME ddr2_dqs_n_fpga;
		else
			ddr2_dqs_n_sdramn <= #DELAY_TIME {DQS_BITS{1'bz}};
	
	always @ *
		if (enable_o == 1'b0)
			ddr2_dq_sdramn <= #DELAY_TIME ddr2_dq_fpga;
		else
			ddr2_dq_sdramn <= #DELAY_TIME {DQ_BITS{1'bz}};
	
	assign ddr2_dqs_fpga = ddr2_dqs_fpgan;
	assign ddr2_dqs_n_fpga = ddr2_dqs_n_fpgan;
	assign ddr2_dq_fpga = ddr2_dq_fpgan;
	assign ddr2_dqs_sdram = ddr2_dqs_sdramn;
	assign ddr2_dqs_n_sdram = ddr2_dqs_n_sdramn;
	assign ddr2_dq_sdram = ddr2_dq_sdramn;

	always @ * begin
		sdram_clk <= #DELAY_TIME ddr2_clk;
		sdram_clkb <= #DELAY_TIME ddr2_clkb;
		sdram_address <= #DELAY_TIME ddr2_address;
		sdram_ba <= #DELAY_TIME ddr2_ba;
		sdram_cke <= #DELAY_TIME ddr2_cke;
		sdram_csb <= #DELAY_TIME ddr2_csb;
		sdram_rasb <= #DELAY_TIME ddr2_rasb;
		sdram_casb <= #DELAY_TIME ddr2_casb;
		sdram_web <= #DELAY_TIME ddr2_web;
		sdram_dmn <= #DELAY_TIME ddr2_dm;
	end
	assign sdram_dm = sdram_dmn;

	ddr2_sdram_cont ddr2_sdram_cont_inst(
		.clk_in(clk),
		.clk_out(sim_clk),
		.reset(reset),
		.input_data(input_data),
		.input_mask(input_mask),
		.read_write(read_write),
		.output_data(output_data),
		.input_address(input_address),
		.addr_fifo_wren(addr_fifo_wren),
		.wrdata_fifo_wren(wrdata_fifo_wren),
		.addr_fifo_full(addr_fifo_full),
		.wrdata_fifo_full(wrdata_fifo_full),
		.rddata_valid(rddata_valid),
		.initialize_end(initialize_end),
		.ddr2_clk(ddr2_clk),
		.ddr2_clkb(ddr2_clkb),
		.ddr2_cke(ddr2_cke),
		.ddr2_dqs(ddr2_dqs_fpga),
		.ddr2_dqs_n(ddr2_dqs_n_fpga),
		.ddr2_dq(ddr2_dq_fpga),
		.ddr2_csb(ddr2_csb),
		.ddr2_rasb(ddr2_rasb),
		.ddr2_casb(ddr2_casb),
		.ddr2_web(ddr2_web),
		.ddr2_dm(ddr2_dm),
		.ddr2_ba(ddr2_ba),
		.ddr2_address(ddr2_address)
	);
		
	ddr2 MT47H16M16_inst(
		.dq(ddr2_dq_sdram),
		.dqs(ddr2_dqs_sdram),
		.dqs_n(ddr2_dqs_n_sdram),
		.rdqs_n(),
		.addr(sdram_address),
		.ba(sdram_ba),
		.ck(sdram_clk),
		.ck_n(sdram_clkb),
		.cke(sdram_cke),
		.cs_n(sdram_csb),
		.ras_n(sdram_rasb),
		.cas_n(sdram_casb),
		.we_n(sdram_web),
		.dm_rdqs(sdram_dm)
	);
	
	initial begin
			reset = 1'b0;
		#1000	reset = 1'b1;
		#300000	reset = 1'b0;
	end
	
	always begin
	   #(CLK_PERIOD/2)	clk = 1'b1 ;
	   #(CLK_PERIOD/2)	clk = 1'b0 ;
	end
	
	task DDR2_WRITE;
		input [USER_INPUT_ADDRESS_WIDTH-1 : 0] address;
		input [INTERFACE_MASK_WIDTH-1 : 0] inmask0;
		input [INTERFACE_DATA_WIDTH-1 : 0] indata0;
		input [INTERFACE_MASK_WIDTH-1 : 0] inmask1;
		input [INTERFACE_DATA_WIDTH-1 : 0] indata1; // DDR24o[Xĝ2Aăf[^
		begin
			input_address = address;
			input_data = indata0;
			input_mask = inmask0;
			read_write = 1'b0; // Write
			addr_fifo_wren = 1'b1;
			wrdata_fifo_wren = 1'b1;
			@(posedge sim_clk); // ̃NbN
			#1; // f^xɂ듮hߍŏx
			addr_fifo_wren = 1'b0; // AhX̂1񂾂
			input_data = indata1;
			input_mask = inmask1;
			@(posedge sim_clk); // ̃NbN
			#1; // f^xɂ듮hߍŏx
			addr_fifo_wren = 1'b0;
			wrdata_fifo_wren = 1'b0;
			#1;
		end
	endtask
		
	task DDR2_READ;
		input [USER_INPUT_ADDRESS_WIDTH-1 : 0] address;
		begin
			input_address = address;
			read_write = 1'b1; // Read
			addr_fifo_wren = 1'b1;
			@(posedge sim_clk); // ̃NbN
			#1; // f^xɂ듮hߍŏx
			addr_fifo_wren = 1'b0;
			#1;
		end
	endtask
	
	initial begin
		input_address = 25'h000_0000;
		input_data = 32'h0000_0000;
		input_mask = 4'b0000;
		read_write = 1'b0;
		addr_fifo_wren = 1'b0;
		wrdata_fifo_wren = 1'b0;

		@(posedge initialize_end); // DDR2 SDRAM̏I܂ő҂
		@(posedge sim_clk);
		#1; // f^xɂ듮hߍŏx
		DDR2_WRITE(25'h000_0100, 4'b0000, 32'h1234_5678, 4'b0000, 32'h9ABC_DEF0);
		DDR2_WRITE(25'h000_0104, 4'b0000, 32'h2233_4455, 4'b0000, 32'h3344_5566);
		DDR2_WRITE(25'h000_0108, 4'b0000, 32'h7788_99AA, 4'b0000, 32'hBBCC_DDEE);
		
		@(posedge sim_clk); // ̃NbN
		@(posedge sim_clk); // ̃NbN
		@(posedge sim_clk); // ̃NbN
	
//		DDR2_WRITE(25'h180_0010, 4'b0000, 32'h1122_3344, 4'b0000, 32'h4455_6677);
		DDR2_WRITE(25'h180_0010, 4'b0000, 32'h5555_5555, 4'b0000, 32'hAAAA_AAAA);
		DDR2_READ(25'h000_0100);
		DDR2_READ(25'h000_0104);
		DDR2_READ(25'h000_0108);
		DDR2_READ(25'h180_0010);
		
		#1000000;
		// ȉ́A킴ƃG[o߂̃eXgx`
		// input_address = 25'h000_0104;
		// read_write = 1'b0; // Write
		// addr_fifo_wren = 1'b1;
		// wrdata_fifo_wren = 1'b1;
		// @(posedge sim_clk); // ̃NbN
		// #1; // f^xɂ듮hߍŏx
		// addr_fifo_wren = 1'b0; // AhX̂1񂾂
		// #1000000;	
		// ܂
		
		$stop;
	end
endmodule

`default_nettype wire
		
	